@@ -9,7 +9,6 @@ Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 # PYRO_LOGFILE    (the name of the logfile if you don't like the default)
 
 import os
-import sys
 import platform
 
 
@@ -18,10 +17,9 @@ class Configuration(object):
                "COMPRESSION", "SERVERTYPE", "DOTTEDNAMES", "COMMTIMEOUT",
                "POLLTIMEOUT", "THREADING2", "ONEWAY_THREADED",
                "DETAILED_TRACEBACK", "SOCK_REUSE", "PREFER_IP_VERSION",
-               "THREADPOOL_MINTHREADS", "THREADPOOL_MAXTHREADS",
-               "THREADPOOL_IDLETIMEOUT", "HMAC_KEY", "AUTOPROXY",
+               "THREADPOOL_SIZE", "HMAC_KEY", "AUTOPROXY",
                "BROADCAST_ADDRS", "NATHOST", "NATPORT", "MAX_MESSAGE_SIZE",
-               "FLAME_ENABLED", "SERIALIZER", "LOGWIRE" )
+               "FLAME_ENABLED", "SERIALIZER", "SERIALIZERS_ACCEPTED", "LOGWIRE" )
 
     def __init__(self):
         self.reset()
@@ -47,9 +45,7 @@ class Configuration(object):
         self.THREADING2 = False    # use threading2 if available?
         self.ONEWAY_THREADED = True     # oneway calls run in their own thread
         self.DETAILED_TRACEBACK = False
-        self.THREADPOOL_MINTHREADS = 4
-        self.THREADPOOL_MAXTHREADS = 50
-        self.THREADPOOL_IDLETIMEOUT = 2.0
+        self.THREADPOOL_SIZE = 16
         self.HMAC_KEY = None   # must be bytes type
         self.AUTOPROXY = True
         self.MAX_MESSAGE_SIZE = 0   # 0 = unlimited
@@ -57,6 +53,7 @@ class Configuration(object):
         self.FLAME_ENABLED = False
         self.PREFER_IP_VERSION = 4    # 4, 6 or 0 (let OS choose according to RFC 3484)
         self.SERIALIZER = "serpent"
+        self.SERIALIZERS_ACCEPTED = "serpent,marshal,json"
         self.LOGWIRE = False   # log wire-level messages
 
         if useenvironment:
@@ -82,6 +79,7 @@ class Configuration(object):
                     setattr(self, symbol, envvalue)
         if self.HMAC_KEY and type(self.HMAC_KEY) is not bytes:
             self.HMAC_KEY = self.HMAC_KEY.encode("utf-8")     # convert to bytes
+        self.SERIALIZERS_ACCEPTED = set(self.SERIALIZERS_ACCEPTED.split(','))
 
     def asDict(self):
         """returns the current config as a regular dictionary"""
