@@ -5,13 +5,13 @@ Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
 from __future__ import with_statement
-import re, struct, sys, time, os
+import re, sys, time, os
 import logging, uuid
-import hashlib, hmac
 from Pyro4 import constants, threadutil, util, socketutil, errors
 from Pyro4.socketserver.threadpoolserver import SocketServer_Threadpool
 from Pyro4.socketserver.multiplexserver import SocketServer_Select, SocketServer_Poll
 from Pyro4 import futures
+from Pyro4.message import Message
 import Pyro4
 
 __all__ = ["URI", "Proxy", "Daemon", "callback", "batch", "async"]
@@ -32,8 +32,11 @@ class URI(object):
 
     There is also a 'Magic format' for simple name resolution using Name server:
       ``PYRONAME:objectname[@location]``  (optional name server location, can also omit location port)
+
+    You can write the protocol in lowercase if you like (``pyro:...``) but it will
+    automatically be converted to uppercase internally.
     """
-    uriRegEx = re.compile(r"(?P<protocol>PYRO[A-Z]*):(?P<object>\S+?)(@(?P<location>\S+))?$")
+    uriRegEx = re.compile(r"(?P<protocol>[Pp][Yy][Rr][Oo][a-zA-Z]*):(?P<object>\S+?)(@(?P<location>\S+))?$")
     __slots__ = ("protocol", "object", "sockname", "host", "port", "object")
 
     def __init__(self, uri):
@@ -47,7 +50,7 @@ class URI(object):
         match=self.uriRegEx.match(uri)
         if not match:
             raise errors.PyroError("invalid uri")
-        self.protocol=match.group("protocol")
+        self.protocol=match.group("protocol").upper()
         self.object=match.group("object")
         location=match.group("location")
         if self.protocol=="PYRONAME":
@@ -158,11 +161,9 @@ class _RemoteMethod(object):
 
 
 def _check_hmac():
-    if Pyro4.config.HMAC_KEY is None or len(Pyro4.config.HMAC_KEY)==0:
-        import warnings
-        warnings.warn("HMAC_KEY not set, protocol data may not be secure")
-    elif sys.version_info>=(3, 0) and type(Pyro4.config.HMAC_KEY) is not bytes:
-        raise errors.PyroError("HMAC_KEY must be bytes type")
+    if Pyro4.config.HMAC_KEY:
+        if sys.version_info>=(3, 0) and type(Pyro4.config.HMAC_KEY) is not bytes:
+            raise errors.PyroError("HMAC_KEY must be bytes type")
 
 
 class Proxy(object):
@@ -175,7 +176,7 @@ class Proxy(object):
     .. automethod:: _pyroBatch
     .. automethod:: _pyroAsync
     """
-    __pyroAttributes=frozenset(["__getnewargs__", "__getinitargs__", "_pyroConnection", "_pyroUri", "_pyroOneway", "_pyroTimeout", "_pyroSeq"])
+    __pyroAttributes = frozenset(["__getnewargs__", "__getnewargs_ex__", "__getinitargs__", "_pyroConnection", "_pyroUri", "_pyroOneway", "_pyroTimeout", "_pyroSeq"])
 
     def __init__(self, uri):
         """
@@ -191,10 +192,13 @@ class Proxy(object):
         self._pyroConnection=None
         self._pyroOneway=set()
         self._pyroSeq=0    # message sequence number
-        self._pyroSerializer = util.get_serializer()
         self.__pyroTimeout=Pyro4.config.COMMTIMEOUT
         self.__pyroLock=threadutil.Lock()
         self.__pyroConnLock=threadutil.Lock()
+        util.get_serializer(Pyro4.config.SERIALIZER)  # assert that the configured serializer is available
+        if os.name=="java" and Pyro4.config.SERIALIZER=="marshal":
+            import warnings
+            warnings.warn("marshal doesn't work correctly with Jython (issue 2077); please choose another serializer", RuntimeWarning)
 
     def __del__(self):
         if hasattr(self, "_pyroConnection"):
@@ -224,7 +228,6 @@ class Proxy(object):
         self._pyroUri, self._pyroOneway, self.__pyroTimeout = state
         self._pyroConnection=None
         self._pyroSeq=0
-        self._pyroSerializer = util.get_serializer()
         self.__pyroLock=threadutil.Lock()
         self.__pyroConnLock=threadutil.Lock()
 
@@ -281,30 +284,35 @@ class Proxy(object):
         if self._pyroConnection is None:
             # rebind here, don't do it from inside the invoke because deadlock will occur
             self.__pyroCreateConnection()
-        data, compressed=self._pyroSerializer.serializeCall(
+        serializer = util.get_serializer(Pyro4.config.SERIALIZER)
+        data, compressed = serializer.serializeCall(
             self._pyroConnection.objectId, methodname, vargs, kwargs,
             compress=Pyro4.config.COMPRESSION)
         if compressed:
-            flags |= MessageFactory.FLAGS_COMPRESSED
+            flags |= Pyro4.message.FLAGS_COMPRESSED
         if methodname in self._pyroOneway:
-            flags |= MessageFactory.FLAGS_ONEWAY
+            flags |= Pyro4.message.FLAGS_ONEWAY
         with self.__pyroLock:
             self._pyroSeq=(self._pyroSeq+1)&0xffff
             if Pyro4.config.LOGWIRE:
-                log.debug("proxy wiredata sending: msgtype=%d flags=0x%x seq=%d data=%r" % (MessageFactory.MSG_INVOKE, flags, self._pyroSeq, data))
-            data=MessageFactory.createMessage(MessageFactory.MSG_INVOKE, data, flags, self._pyroSeq)
+                log.debug("proxy wiredata sending: msgtype=%d flags=0x%x ser=%d seq=%d data=%r" % (Pyro4.message.MSG_INVOKE, flags, serializer.serializer_id, self._pyroSeq, data))
+            msg = Message(Pyro4.message.MSG_INVOKE, data, serializer.serializer_id, flags, self._pyroSeq)
             try:
-                self._pyroConnection.send(data)
-                del data  # invite GC to collect the object, don't wait for out-of-scope
-                if flags & MessageFactory.FLAGS_ONEWAY:
+                self._pyroConnection.send(msg.to_bytes())
+                del msg  # invite GC to collect the object, don't wait for out-of-scope
+                if flags & Pyro4.message.FLAGS_ONEWAY:
                     return None    # oneway call, no response data
                 else:
-                    msgType, flags, seq, data = MessageFactory.getMessage(self._pyroConnection, [MessageFactory.MSG_RESULT])
+                    msg = Message.recv(self._pyroConnection, [Pyro4.message.MSG_RESULT])
                     if Pyro4.config.LOGWIRE:
-                        log.debug("proxy wiredata received: msgtype=%d flags=0x%x seq=%d data=%r" % (msgType, flags, seq, data) )
-                    self.__pyroCheckSequence(seq)
-                    data=self._pyroSerializer.deserializeData(data, compressed=flags & MessageFactory.FLAGS_COMPRESSED)
-                    if flags & MessageFactory.FLAGS_EXCEPTION:
+                        log.debug("proxy wiredata received: msgtype=%d flags=0x%x ser=%d seq=%d data=%r" % (msg.type, msg.flags, msg.serializer_id, msg.seq, msg.data) )
+                    self.__pyroCheckSequence(msg.seq)
+                    if msg.serializer_id != serializer.serializer_id:
+                        error = "invalid serializer in response: %d" % msg.serializer_id
+                        log.error(error)
+                        raise errors.ProtocolError(error)
+                    data = serializer.deserializeData(msg.data, compressed=msg.flags & Pyro4.message.FLAGS_COMPRESSED)
+                    if msg.flags & Pyro4.message.FLAGS_EXCEPTION:
                         if sys.platform=="cli":
                             util.fixIronPythonExceptionForPickle(data, False)
                         raise data
@@ -347,7 +355,7 @@ class Proxy(object):
                     sock=socketutil.createSocket(connect=connect_location, reuseaddr=Pyro4.config.SOCK_REUSE, timeout=self.__pyroTimeout)
                     conn=socketutil.SocketConnection(sock, uri.object)
                     # Do handshake. For now, no need to send anything. (message type CONNECT is not yet used)
-                    msgType, flags, seq, data = MessageFactory.getMessage(conn, None)
+                    msg = Message.recv(conn, None)
                     # any trailing data (dataLen>0) is an error message, if any
                 except Exception:
                     x=sys.exc_info()[1]
@@ -358,27 +366,28 @@ class Proxy(object):
                     if isinstance(x, errors.CommunicationError):
                         raise
                     else:
-                        raise errors.CommunicationError(err)
+                        ce = errors.CommunicationError(err)
+                        ce.__cause__ = x
+                        raise ce
                 else:
-                    if msgType==MessageFactory.MSG_CONNECTFAIL:
+                    if msg.type==Pyro4.message.MSG_CONNECTFAIL:
                         error="connection rejected"
-                        if data:
-                            if sys.version_info>=(3, 0):
-                                data=str(data, "utf-8")
-                            error+=", reason: " + data
+                        if msg.data:
+                            serializer = util.get_serializer_by_id(msg.serializer_id)
+                            data = serializer.deserializeData(msg.data, compressed=msg.flags & Pyro4.message.FLAGS_COMPRESSED)
+                            error += ", reason: " + data
                         conn.close()
                         log.error(error)
                         raise errors.CommunicationError(error)
-                    elif msgType==MessageFactory.MSG_CONNECTOK:
+                    elif msg.type==Pyro4.message.MSG_CONNECTOK:
                         self._pyroConnection=conn
                         if replaceUri:
-                            log.debug("replacing uri with bound one")
                             self._pyroUri=uri
                         log.debug("connected to %s", self._pyroUri)
                         return True
                     else:
                         conn.close()
-                        err="connect: invalid msg type %d received" % msgType
+                        err="connect: invalid msg type %d received" % msg.type
                         log.error(err)
                         raise errors.ProtocolError(err)
 
@@ -406,9 +415,9 @@ class Proxy(object):
         return _AsyncProxyAdapter(self)
 
     def _pyroInvokeBatch(self, calls, oneway=False):
-        flags=MessageFactory.FLAGS_BATCH
+        flags=Pyro4.message.FLAGS_BATCH
         if oneway:
-            flags|=MessageFactory.FLAGS_ONEWAY
+            flags|=Pyro4.message.FLAGS_ONEWAY
         return self._pyroInvoke("<batch>", calls, None, flags)
 
 
@@ -518,86 +527,6 @@ def async(proxy):
     return proxy._pyroAsync()
 
 
-class MessageFactory(object):
-    """internal helper class to construct Pyro protocol messages"""
-    headerFmt = '!4sHHHHiH20s'    # header (id, version, msgtype, flags, sequencenumber, dataLen, checksum, hmac)
-    # note: the sequencenumber is used to check if response messages correspond to the
-    # actual request message. This prevents the situation where Pyro would perhaps return
-    # the response data from another remote call (which would not result in an error otherwise!)
-    # This could happen for instance if the socket data stream gets out of sync, perhaps due To
-    # some form of signal that interrupts I/O.
-    # The header checksum is a simple sum of the header fields to make reasonably sure
-    # that we are dealing with an actual correct PYRO protocol header and not some random
-    # data that happens to start with the 'PYRO' protocol identifier.
-    HEADERSIZE=struct.calcsize(headerFmt)
-    MSG_CONNECT = 1
-    MSG_CONNECTOK = 2
-    MSG_CONNECTFAIL = 3
-    MSG_INVOKE = 4
-    MSG_RESULT = 5
-    MSG_PING = 6
-    FLAGS_EXCEPTION = 1<<0
-    FLAGS_COMPRESSED = 1<<1
-    FLAGS_ONEWAY = 1<<2
-    FLAGS_HMAC = 1<<3
-    FLAGS_BATCH = 1<<4
-    MAGIC = 0x34E9
-    pyro_tag = b"PYRO"
-    empty_hmac = b"\0"*hashlib.sha1().digest_size
-
-    @classmethod
-    def createMessage(cls, msgType, databytes, flags, seq):
-        """creates a message containing a header followed by the given databytes"""
-        databytes=databytes or b""
-        if 0 < Pyro4.config.MAX_MESSAGE_SIZE < len(databytes):
-            raise errors.ProtocolError("max message size exceeded (%d where max=%d)" % (len(databytes), Pyro4.config.MAX_MESSAGE_SIZE))
-        if Pyro4.config.HMAC_KEY:
-            flags|=MessageFactory.FLAGS_HMAC
-            bodyhmac=hmac.new(Pyro4.config.HMAC_KEY, databytes, digestmod=hashlib.sha1).digest()
-        else:
-            bodyhmac=MessageFactory.empty_hmac
-        headerchecksum=(msgType+constants.PROTOCOL_VERSION+len(databytes)+flags+seq+MessageFactory.MAGIC)&0xffff
-        msg=struct.pack(cls.headerFmt, cls.pyro_tag, constants.PROTOCOL_VERSION, msgType, flags, seq, len(databytes), headerchecksum, bodyhmac)
-        return msg+databytes
-
-    @classmethod
-    def parseMessageHeader(cls, headerData):
-        """Parses a message header. Returns a tuple of messagetype, messageflags, sequencenumber, datalength, datahmac."""
-        if not headerData or len(headerData)!=cls.HEADERSIZE:
-            raise errors.ProtocolError("header data size mismatch")
-        tag, ver, msgType, flags, seq, dataLen, headerchecksum, datahmac = struct.unpack(cls.headerFmt, headerData)
-        if tag!=cls.pyro_tag or ver!=constants.PROTOCOL_VERSION:
-            raise errors.ProtocolError("invalid data or unsupported protocol version")
-        if headerchecksum!=(msgType+ver+dataLen+flags+seq+MessageFactory.MAGIC)&0xffff:
-            raise errors.ProtocolError("header checksum mismatch")
-        return msgType, flags, seq, dataLen, datahmac
-
-    @classmethod
-    def getMessage(cls, connection, requiredMsgTypes):
-        headerdata = connection.recv(cls.HEADERSIZE)
-        msgType, flags, seq, datalen, datahmac = cls.parseMessageHeader(headerdata)
-        if 0 < Pyro4.config.MAX_MESSAGE_SIZE < datalen:
-            errorMsg = "max message size exceeded (%d where max=%d)" % (datalen, Pyro4.config.MAX_MESSAGE_SIZE)
-            log.error("connection "+str(connection)+": "+errorMsg)
-            connection.close()   # close the socket because at this point we can't return the correct sequence number for returning an error message
-            raise errors.ProtocolError(errorMsg)
-        if requiredMsgTypes and msgType not in requiredMsgTypes:
-            err = "invalid msg type %d received" % msgType
-            log.error(err)
-            raise errors.ProtocolError(err)
-        databytes=connection.recv(datalen)
-        local_hmac_set=Pyro4.config.HMAC_KEY is not None and len(Pyro4.config.HMAC_KEY) > 0
-        if flags&MessageFactory.FLAGS_HMAC and local_hmac_set:
-            if datahmac != hmac.new(Pyro4.config.HMAC_KEY, databytes, digestmod=hashlib.sha1).digest():
-                raise errors.SecurityError("message hmac mismatch")
-        elif flags&MessageFactory.FLAGS_HMAC != local_hmac_set:
-            # Message contains hmac and local HMAC_KEY not set, or vice versa. This is not allowed.
-            err="hmac key config not symmetric"
-            log.warning(err)
-            raise errors.SecurityError(err)
-        return msgType, flags, seq, databytes
-
-
 def pyroObjectToAutoProxy(self):
     """reduce function that automatically replaces Pyro objects by a Proxy"""
     if Pyro4.config.AUTOPROXY:
@@ -644,8 +573,9 @@ class Daemon(object):
         elif Pyro4.config.SERVERTYPE=="multiplex":
             # choose the 'best' multiplexing implementation
             if os.name=="java":
-                raise NotImplementedError("select or poll-based server is not supported for jython, use thread server instead")
-            self.transportServer = SocketServer_Poll() if socketutil.hasPoll else SocketServer_Select()
+                self.transportServer = SocketServer_Select()    # poll doesn't work as given in jython ('socket must be in nonblocking mode')
+            else:
+                self.transportServer = SocketServer_Poll() if socketutil.hasPoll else SocketServer_Select()
         else:
             raise errors.PyroError("invalid server type '%s'" % Pyro4.config.SERVERTYPE)
         self.transportServer.init(self, host, port, unixsocket)
@@ -660,7 +590,6 @@ class Daemon(object):
         self.natLocationStr = "%s:%d" % (nathost, natport_for_loc) if nathost else None
         if self.natLocationStr:
             log.debug("NAT address is %s", self.natLocationStr)
-        self.serializer=util.get_serializer()
         pyroObject=DaemonObject(self)
         pyroObject._pyroId=constants.DAEMON_NAME
         #: Dictionary from Pyro object id to the actual Pyro object registered by this id
@@ -668,6 +597,9 @@ class Daemon(object):
         self.__mustshutdown=threadutil.Event()
         self.__loopstopped=threadutil.Event()
         self.__loopstopped.set()
+        # assert that the configured serializers are available, and remember their ids:
+        self.__serializer_ids = set([util.get_serializer(ser_name).serializer_id for ser_name in Pyro4.config.SERIALIZERS_ACCEPTED])
+        log.debug("accepted serializers: %s" % Pyro4.config.SERIALIZERS_ACCEPTED)
 
     @property
     def sock(self):
@@ -742,8 +674,11 @@ class Daemon(object):
         # We need a minimal amount of data or the socket will remain blocked
         # on some systems... (messages smaller than 40 bytes)
         # Return True for successful handshake, False if something was wrong.
-        msg=MessageFactory.createMessage(MessageFactory.MSG_CONNECTOK, b"ok", 0, 1)
-        conn.send(msg)
+        # We default to the marshal serializer to send message payload of "ok"
+        ser = util.get_serializer("marshal")
+        data = ser.dumps("ok")
+        msg = Message(Pyro4.message.MSG_CONNECTOK, data, ser.serializer_id, 0, 1)
+        conn.send(msg.to_bytes())
         return True
 
     def handleRequest(self, conn):
@@ -752,27 +687,36 @@ class Daemon(object):
         wraps it in a reply to the calling side, as to not make this server side loop
         terminate due to exceptions caused by remote invocations.
         """
-        flags=0
-        seq=0
+        request_flags=0
+        request_seq=0
+        request_serializer_id = util.MarshalSerializer.serializer_id
         wasBatched=False
         isCallback=False
         try:
-            msgType, flags, seq, data = MessageFactory.getMessage(conn, [MessageFactory.MSG_INVOKE, MessageFactory.MSG_PING])
+            msg = Message.recv(conn, [Pyro4.message.MSG_INVOKE, Pyro4.message.MSG_PING])
+            request_flags = msg.flags
+            request_seq = msg.seq
+            request_serializer_id = msg.serializer_id
             if Pyro4.config.LOGWIRE:
-                log.debug("daemon wiredata received: msgtype=%d flags=0x%x seq=%d data=%r" % (msgType, flags, seq, data) )
-            if msgType == MessageFactory.MSG_PING:
-                # return same seq, but ignore any data (it's a ping, not an echo)
-                msg = MessageFactory.createMessage(MessageFactory.MSG_PING, None, 0, seq)
-                conn.send(msg)
+                log.debug("daemon wiredata received: msgtype=%d flags=0x%x ser=%d seq=%d data=%r" % (msg.type, msg.flags, msg.serializer_id, msg.seq, msg.data) )
+            if msg.type == Pyro4.message.MSG_PING:
+                # return same seq, but ignore any data (it's a ping, not an echo). Nothing is deserialized.
+                msg = Message(Pyro4.message.MSG_PING, b"pong", msg.serializer_id, 0, msg.seq)
+                if Pyro4.config.LOGWIRE:
+                    log.debug("daemon wiredata sending: msgtype=%d flags=0x%x ser=%d seq=%d data=%r" % (msg.type, msg.flags, msg.serializer_id, msg.seq, msg.data))
+                conn.send(msg.to_bytes())
                 return
-            objId, method, vargs, kwargs = self.serializer.deserializeCall(data, compressed=flags & MessageFactory.FLAGS_COMPRESSED)
-            del data  # invite GC to collect the object, don't wait for out-of-scope
-            obj=self.objectsById.get(objId)
+            if msg.serializer_id not in self.__serializer_ids:
+                raise errors.ProtocolError("message used serializer that is not accepted: %d" % msg.serializer_id)
+            serializer = util.get_serializer_by_id(msg.serializer_id)
+            objId, method, vargs, kwargs = serializer.deserializeCall(msg.data, compressed=msg.flags & Pyro4.message.FLAGS_COMPRESSED)
+            del msg  # invite GC to collect the object, don't wait for out-of-scope
+            obj = self.objectsById.get(objId)
             if obj is not None:
                 if kwargs and sys.version_info<(2, 6, 5) and os.name!="java":
                     # Python before 2.6.5 doesn't accept unicode keyword arguments
                     kwargs = dict((str(k), kwargs[k]) for k in kwargs)
-                if flags & MessageFactory.FLAGS_BATCH:
+                if request_flags & Pyro4.message.FLAGS_BATCH:
                     # batched method calls, loop over them all and collect all results
                     data=[]
                     for method, vargs, kwargs in vargs:
@@ -793,7 +737,7 @@ class Daemon(object):
                 else:
                     # normal single method call
                     method=util.resolveDottedAttribute(obj, method, Pyro4.config.DOTTEDNAMES)
-                    if flags & MessageFactory.FLAGS_ONEWAY and Pyro4.config.ONEWAY_THREADED:
+                    if request_flags & Pyro4.message.FLAGS_ONEWAY and Pyro4.config.ONEWAY_THREADED:
                         # oneway call to be run inside its own thread
                         thread=threadutil.Thread(target=method, args=vargs, kwargs=kwargs)
                         thread.setDaemon(True)
@@ -804,38 +748,39 @@ class Daemon(object):
             else:
                 log.debug("unknown object requested: %s", objId)
                 raise errors.DaemonError("unknown object")
-            if flags & MessageFactory.FLAGS_ONEWAY:
+            if request_flags & Pyro4.message.FLAGS_ONEWAY:
                 return   # oneway call, don't send a response
             else:
-                data, compressed=self.serializer.serializeData(data, compress=Pyro4.config.COMPRESSION)
-                flags=0
+                data, compressed = serializer.serializeData(data, compress=Pyro4.config.COMPRESSION)
+                response_flags=0
                 if compressed:
-                    flags |= MessageFactory.FLAGS_COMPRESSED
+                    response_flags |= Pyro4.message.FLAGS_COMPRESSED
                 if wasBatched:
-                    flags |= MessageFactory.FLAGS_BATCH
+                    response_flags |= Pyro4.message.FLAGS_BATCH
                 if Pyro4.config.LOGWIRE:
-                    log.debug("daemon wiredata sending: msgtype=%d flags=0x%x seq=%d data=%r" % (MessageFactory.MSG_RESULT, flags, seq, data))
-                msg=MessageFactory.createMessage(MessageFactory.MSG_RESULT, data, flags, seq)
-                del data
-                conn.send(msg)
+                    log.debug("daemon wiredata sending: msgtype=%d flags=0x%x ser=%d seq=%d data=%r" % (Pyro4.message.MSG_RESULT, response_flags, serializer.serializer_id, request_seq, data))
+                msg = Message(Pyro4.message.MSG_RESULT, data, serializer.serializer_id, response_flags, request_seq)
+                conn.send(msg.to_bytes())
         except Exception:
             xt, xv = sys.exc_info()[0:2]
             if xt is not errors.ConnectionClosedError:
                 log.debug("Exception occurred while handling request: %r", xv)
-                if not flags & MessageFactory.FLAGS_ONEWAY:
-                    # only return the error to the client if it wasn't a oneway call
+                if not request_flags & Pyro4.message.FLAGS_ONEWAY and not isinstance(xv, errors.CommunicationError):
+                    # only return the error to the client if it wasn't a oneway call, and not a communication error
+                    # (in these cases, it makes no sense to try to report the error back to the client...)
                     tblines=util.formatTraceback(detailed=Pyro4.config.DETAILED_TRACEBACK)
-                    self._sendExceptionResponse(conn, seq, xv, tblines)
+                    self._sendExceptionResponse(conn, request_seq, request_serializer_id, xv, tblines)
             if isCallback or isinstance(xv, (errors.CommunicationError, errors.SecurityError)):
                 raise       # re-raise if flagged as callback, communication or security error.
 
-    def _sendExceptionResponse(self, connection, seq, exc_value, tbinfo):
+    def _sendExceptionResponse(self, connection, seq, serializer_id, exc_value, tbinfo):
         """send an exception back including the local traceback info"""
         exc_value._pyroTraceback=tbinfo
         if sys.platform=="cli":
             util.fixIronPythonExceptionForPickle(exc_value, True)  # piggyback attributes
+        serializer = util.get_serializer_by_id(serializer_id)
         try:
-            data, compressed =self.serializer.serializeData(exc_value)
+            data, compressed = serializer.serializeData(exc_value)
         except:
             # the exception object couldn't be serialized, use a generic PyroError instead
             xt, xv, tb = sys.exc_info()
@@ -844,13 +789,14 @@ class Daemon(object):
             exc_value._pyroTraceback=tbinfo
             if sys.platform=="cli":
                 util.fixIronPythonExceptionForPickle(exc_value, True)  # piggyback attributes
-            data, compressed =self.serializer.serializeData(exc_value)
-        flags = MessageFactory.FLAGS_EXCEPTION
+            data, compressed = serializer.serializeData(exc_value)
+        flags = Pyro4.message.FLAGS_EXCEPTION
         if compressed:
-            flags |= MessageFactory.FLAGS_COMPRESSED
-        msg=MessageFactory.createMessage(MessageFactory.MSG_RESULT, data, flags, seq)
-        del data
-        connection.send(msg)
+            flags |= Pyro4.message.FLAGS_COMPRESSED
+        if Pyro4.config.LOGWIRE:
+            log.debug("daemon wiredata sending (error response): msgtype=%d flags=0x%x ser=%d seq=%d data=%r" % (Pyro4.message.MSG_RESULT, flags, serializer.serializer_id, seq, data))
+        msg = Message(Pyro4.message.MSG_RESULT, data, serializer.serializer_id, flags, seq)
+        connection.send(msg.to_bytes())
 
     def register(self, obj, objectId=None):
         """
@@ -872,7 +818,9 @@ class Daemon(object):
         obj._pyroDaemon=self
         if Pyro4.config.AUTOPROXY:
             # register a custom serializer for the type to automatically return proxies
-            self.serializer.register_type_replacement(type(obj), pyroObjectToAutoProxy)
+            # we need to do this for all known serializers
+            for ser in util._serializers.values():
+                ser.register_type_replacement(type(obj), pyroObjectToAutoProxy)
         # register the object in the mapping
         self.objectsById[obj._pyroId]=obj
         return self.uriFor(objectId)
@@ -941,7 +889,7 @@ class Daemon(object):
         self.close()
 
     def __getstate__(self):
-        return {}   # a little hack to make it possible to serialize Pyro objects.
+        return {}   # a little hack to make it possible to serialize Pyro objects, because they can reference a daemon
 
     def __getstate_for_dict__(self):
         return self.__getstate__()
