@@ -55,6 +55,8 @@ class NameServer(threadutil.Thread):
     def run(self):
         self.uri, self.ns_daemon, self.bc_server = naming.startNS(self.hostname)
         self.started.set()
+        if self.bc_server:
+            self.bc_server.runInThread()
         self.ns_daemon.requestLoop()
 
 
@@ -82,8 +84,7 @@ def main(args, returnWithoutLooping=False):
         options.quiet=False
     if not options.quiet:
         print("Starting Pyro's built-in test echo server.")
-    if os.name!="java":
-        Pyro4.config.SERVERTYPE="multiplex"
+    Pyro4.config.SERVERTYPE="multiplex"
 
     hmac = (options.key or "").encode("utf-8")
     Pyro4.config.HMAC_KEY=hmac or Pyro4.config.HMAC_KEY
@@ -108,6 +109,11 @@ def main(args, returnWithoutLooping=False):
         ns.register(objectName, uri)
         if options.verbose:
             print("using name server at %s" % ns._pyroUri)
+            if nameserver is not None:
+                if nameserver.bc_server:
+                    print("broadcast server running at %s" % nameserver.bc_server.locationStr)
+                else:
+                    print("not using a broadcast server")
     else:
         if options.verbose:
             print("not using a name server.")
