@@ -67,7 +67,7 @@ class FlameModule(object):
         self.module = module
 
     def __getattr__(self, item):
-        if item in ("__getnewargs__", "__getinitargs__"):
+        if item in ("__getnewargs__", "__getnewargs_ex__", "__getinitargs__"):
             raise AttributeError(item)
         return Pyro4.core._RemoteMethod(self.__invoke, "%s.%s" % (self.module, item))
 
@@ -183,8 +183,8 @@ class Flame(object):
     Be *very* cautious before starting this: it allows the clients full access to everything on your system.
     """
     def __init__(self):
-        if Pyro4.config.SERIALIZER != "pickle":
-            raise RuntimeError("flame requires the pickle serializer")
+        if "pickle" not in Pyro4.config.SERIALIZERS_ACCEPTED:
+            raise RuntimeError("flame requires the pickle serializer to be enabled")
 
     def module(self, name):
         """import a module on the server given by the module name and returns a proxy to it"""
