@@ -6,11 +6,12 @@ Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 
 from __future__ import with_statement
 import time
+import os
 import Pyro4.core
 import Pyro4.naming
 import Pyro4.socketutil
 import Pyro4.constants
-from Pyro4.errors import NamingError
+from Pyro4.errors import CommunicationError, NamingError
 from Pyro4 import threadutil
 from testsupport import *
 
@@ -24,7 +25,10 @@ class NSLoopThread(threadutil.Thread):
         self.running.clear()
     def run(self):
         self.running.set()
-        self.nameserver.requestLoop()
+        try:
+            self.nameserver.requestLoop()
+        except CommunicationError:
+            pass   # ignore pyro communication errors
 
 class BCSetupTests(unittest.TestCase):
     def setUp(self):
@@ -55,6 +59,7 @@ class NameServerTests(unittest.TestCase):
         self.daemonthread=NSLoopThread(self.nameserver)
         self.daemonthread.start()
         self.daemonthread.running.wait()
+        time.sleep(0.05)
         self.old_bcPort=Pyro4.config.NS_BCPORT
         self.old_nsPort=Pyro4.config.NS_PORT
         self.old_nsHost=Pyro4.config.NS_HOST
@@ -190,7 +195,10 @@ class NameServerTests0000(unittest.TestCase):
         Pyro4.config.POLLTIMEOUT=0.1
         Pyro4.config.HMAC_KEY=b"testsuite"
         self.nsUri, self.nameserver, self.bcserver = Pyro4.naming.startNS(host="", port=0, bcport=0)
-        self.assertEqual("0.0.0.0", self.nsUri.host, "for hostname \"\" the resulting ip must be 0.0.0.0")
+        host_check = self.nsUri.host
+        if host_check == "0:0:0:0:0:0:0:0":   # this happens on jython
+            host_check = "0.0.0.0"
+        self.assertEqual("0.0.0.0", host_check, "for hostname \"\" the resulting ip must be 0.0.0.0 (or ipv6 equivalent)")
         self.assertTrue(self.bcserver is not None,"expected a BC server to be running")
         self.bcserver.runInThread()
         self.old_bcPort=Pyro4.config.NS_BCPORT
@@ -208,6 +216,7 @@ class NameServerTests0000(unittest.TestCase):
         Pyro4.config.NS_BCPORT=self.old_bcPort
         Pyro4.config.HMAC_KEY=None
 
+    @unittest.skipUnless(os.name!="java", "jython does strange things with bc server on 0.0.0.0 (it hangs)")
     def testBCLookup0000(self):
         ns=Pyro4.naming.locateNS()  # broadcast lookup
         self.assertTrue(isinstance(ns, Pyro4.core.Proxy))
