@@ -8,6 +8,7 @@ from __future__ import with_statement
 import Pyro4.core
 import Pyro4.errors
 import Pyro4.util
+import Pyro4.message
 import time, os, sys, platform
 from Pyro4 import threadutil
 from testsupport import *
@@ -33,6 +34,8 @@ class MyThing(object):
         time.sleep(delay)
         return "slept for "+str(id)
     def testargs(self,x,*args,**kwargs):
+        if kwargs:
+            key=list(kwargs.keys())[0]
         return [x, list(args), kwargs]    # don't return tuples, this enables us to test json serialization as well.
     def nonserializableException(self):
         raise NonserializableError(("xantippe", lambda x: 0))
@@ -49,21 +52,23 @@ class DaemonLoopThread(threadutil.Thread):
         self.running.set()
         try:
             self.pyrodaemon.requestLoop()
-        except:
-            print("Swallow exception from terminated daemon")
+        except Pyro4.errors.CommunicationError:
+            pass    # ignore pyro communication errors
 
 
 class DaemonWithSabotagedHandshake(Pyro4.core.Daemon):
     def _handshake(self, conn):
         # a bit of a hack, overriding this internal method to return a CONNECTFAIL...
-        msg=Pyro4.core.MessageFactory.createMessage(Pyro4.core.MessageFactory.MSG_CONNECTFAIL, b"rigged connection failure", 0, 1)
-        conn.send(msg)
+        serializer = Pyro4.util.get_serializer("marshal")
+        data, _ = serializer.serializeData("rigged connection failure", compress=False)
+        msg = Pyro4.message.Message(Pyro4.message.MSG_CONNECTFAIL, data, serializer.serializer_id, 0, 1)
+        conn.send(msg.to_bytes())
         return False
     
 class ServerTestsBrokenHandshake(unittest.TestCase):
     def setUp(self):
         Pyro4.config.HMAC_KEY=b"testsuite"
-        Pyro4.config.SERIALIZER="pickle"
+        Pyro4.config.SERIALIZERS_ACCEPTED.add("pickle")
         self.daemon=DaemonWithSabotagedHandshake(port=0)
         obj=MyThing()
         uri=self.daemon.register(obj, "something")
@@ -71,12 +76,13 @@ class ServerTestsBrokenHandshake(unittest.TestCase):
         self.daemonthread=DaemonLoopThread(self.daemon)
         self.daemonthread.start()
         self.daemonthread.running.wait()
+        time.sleep(0.05)
     def tearDown(self):
         time.sleep(0.05)
         self.daemon.shutdown()
         self.daemonthread.join()
         Pyro4.config.HMAC_KEY=None
-        Pyro4.config.SERIALIZER="serpent"
+        Pyro4.config.SERIALIZERS_ACCEPTED.discard("pickle")
     def testDaemonConnectFail(self):
         # check what happens when the daemon responds with a failed connection msg
         with Pyro4.Proxy(self.objectUri) as p:
@@ -86,13 +92,14 @@ class ServerTestsBrokenHandshake(unittest.TestCase):
             except Pyro4.errors.CommunicationError:
                 xv=sys.exc_info()[1]
                 message=str(xv)
+                self.assertTrue("reason:" in message)
                 self.assertTrue("rigged connection failure" in message)
 
 class ServerTestsOnce(unittest.TestCase):
     """tests that are fine to run with just a single server type"""
     def setUp(self):
         Pyro4.config.HMAC_KEY=b"testsuite"
-        Pyro4.config.SERIALIZER="pickle"
+        Pyro4.config.SERIALIZERS_ACCEPTED.add("pickle")
         self.daemon=Pyro4.core.Daemon(port=0)
         obj=MyThing()
         uri=self.daemon.register(obj, "something")
@@ -100,24 +107,25 @@ class ServerTestsOnce(unittest.TestCase):
         self.daemonthread=DaemonLoopThread(self.daemon)
         self.daemonthread.start()
         self.daemonthread.running.wait()
+        time.sleep(0.05)
     def tearDown(self):
         time.sleep(0.05)
         if self.daemon is not None:
             self.daemon.shutdown()
             self.daemonthread.join()
         Pyro4.config.HMAC_KEY=None
-        Pyro4.config.SERIALIZER="serpent"
+        Pyro4.config.SERIALIZERS_ACCEPTED.discard("pickle")
 
     def testPingMessage(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
             p._pyroBind()
             conn = p._pyroConnection
-            data = Pyro4.core.MessageFactory.createMessage(Pyro4.core.MessageFactory.MSG_PING, b"abc", 0, 999)
-            conn.send(data)
-            msgType, flags, seq, data = Pyro4.core.MessageFactory.getMessage(conn, [Pyro4.core.MessageFactory.MSG_PING])
-            self.assertEqual(Pyro4.core.MessageFactory.MSG_PING, msgType)
-            self.assertEqual(999, seq)
-            self.assertEqual(b"", data)
+            msg = Pyro4.message.Message(Pyro4.message.MSG_PING, b"something", 42, 0, 999)
+            conn.send(msg.to_bytes())
+            msg = Pyro4.message.Message.recv(conn, [Pyro4.message.MSG_PING])
+            self.assertEqual(Pyro4.message.MSG_PING, msg.type)
+            self.assertEqual(999, msg.seq)
+            self.assertEqual(b"pong", msg.data)
 
     def testNoDottedNames(self):
         Pyro4.config.DOTTEDNAMES=False
@@ -138,14 +146,16 @@ class ServerTestsOnce(unittest.TestCase):
             self.assertEqual([1,[],{}], p.testargs(1))
             self.assertEqual([1,[2,3],{'a':4}], p.testargs(1,2,3,a=4))
             self.assertEqual([1,[],{'a':2}], p.testargs(1, **{'a':2}))
-            if sys.version_info>=(2,6,5):
-                # python 2.6.5 and later support unicode keyword args
-                self.assertEqual([1,[],{unichr(65):2}], p.testargs(1, **{unichr(65):2}))
-                if platform.python_implementation()!="PyPy":
-                    # PyPy doesn't accept unicode kwargs that cannot be encoded to ascii, see https://bugs.pypy.org/issue751
-                    result=p.testargs(1, **{unichr(0x20ac):2})
-                    key=list(result[2].keys())[0]
-                    self.assertTrue(key==unichr(0x20ac))
+
+    @unittest.skipUnless(sys.version_info>=(2,6,5), "unicode kwargs needs 2.6.5 or newer")
+    def testUnicodeKwargs(self):
+        with Pyro4.core.Proxy(self.objectUri) as p:
+            self.assertEqual([1,[],{unichr(65):2}], p.testargs(1, **{unichr(65):2}))
+            result=p.testargs(unichr(0x20ac), **{unichr(0x20ac):2})
+            self.assertEqual(result[0], unichr(0x20ac))
+            key=list(result[2].keys())[0]
+            self.assertTrue(type(key) is unicode)
+            self.assertEqual(key, unichr(0x20ac))
 
     def testDottedNames(self):
         try:
@@ -186,28 +196,25 @@ class ServerTestsOnce(unittest.TestCase):
                 xt, xv, tb = sys.exc_info()
                 self.assertTrue(issubclass(xt, Pyro4.errors.PyroError))
                 tblines = "\n".join(Pyro4.util.getPyroTraceback())
-                self.assertTrue("PyroError: Error serializing exception" in tblines)
-                s1 = "Original exception: <class 'testsupport.NonserializableError'>:"
-                s2 = "Original exception: <class 'PyroTests.testsupport.NonserializableError'>:"
-                self.assertTrue(s1 in tblines or s2 in tblines)
-                self.assertTrue("raise NonserializableError((\"xantippe" in tblines)
+                self.assertTrue("unsupported serialized class" in tblines)
 
     def testNonserializableException_pickle(self):
-        if Pyro4.config.SERIALIZER!="pickle":
-            self.skipTest("only for pickle-serializer")
         with Pyro4.core.Proxy(self.objectUri) as p:
+            Pyro4.config.SERIALIZER = "pickle"
             try:
                 p.nonserializableException()
                 self.fail("should crash")
             except Exception:
                 xt, xv, tb = sys.exc_info()
-                self.assertEqual(xt, Pyro4.errors.PyroError)
+                self.assertTrue(issubclass(xt, Pyro4.errors.PyroError))
                 tblines = "\n".join(Pyro4.util.getPyroTraceback())
                 self.assertTrue("PyroError: Error serializing exception" in tblines)
                 s1 = "Original exception: <class 'testsupport.NonserializableError'>:"
                 s2 = "Original exception: <class 'PyroTests.testsupport.NonserializableError'>:"
                 self.assertTrue(s1 in tblines or s2 in tblines)
                 self.assertTrue("raise NonserializableError((\"xantippe" in tblines)
+            finally:
+                Pyro4.config.SERIALIZER="serpent"
 
     def testBatchProxy(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
@@ -335,6 +342,7 @@ class ServerTestsOnce(unittest.TestCase):
 
     def testAutoProxy(self):
         obj=MyThing2()
+        Pyro4.config.SERIALIZER="pickle"
         try:
             with Pyro4.core.Proxy(self.objectUri) as p:
                 Pyro4.config.AUTOPROXY=False   # make sure autoproxy is disabled
@@ -357,9 +365,9 @@ class ServerTestsOnce(unittest.TestCase):
                 self.assertTrue(isinstance(result,MyThing2), "unregistered pyro object must be normal class again")
                 # note: the custom serializer may still be active but it should be smart enough to see
                 # that the object is no longer a pyro object, and therefore, no proxy should be created.
-
         finally:
             Pyro4.config.AUTOPROXY=True
+            Pyro4.config.SERIALIZER="serpent"
 
     def testConnectOnce(self):
         with Pyro4.core.Proxy(self.objectUri) as proxy:
@@ -426,10 +434,8 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
         Pyro4.config.POLLTIMEOUT=0.1
         Pyro4.config.SERVERTYPE=self.SERVERTYPE
         Pyro4.config.COMMTIMEOUT=self.COMMTIMEOUT
-        Pyro4.config.THREADPOOL_MINTHREADS=10
-        Pyro4.config.THREADPOOL_MAXTHREADS=20
         Pyro4.config.HMAC_KEY=b"testsuite"
-        Pyro4.config.SERIALIZER="pickle"
+        Pyro4.config.SERIALIZERS_ACCEPTED.add("pickle")
         self.daemon=Pyro4.core.Daemon(port=0)
         obj=MyThing()
         uri=self.daemon.register(obj, "something")
@@ -437,6 +443,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
         self.daemonthread=DaemonLoopThread(self.daemon)
         self.daemonthread.start()
         self.daemonthread.running.wait()
+        time.sleep(0.05)
     def tearDown(self):
         time.sleep(0.05)
         self.daemon.shutdown()
@@ -444,7 +451,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
         Pyro4.config.SERVERTYPE="thread"
         Pyro4.config.COMMTIMEOUT=None
         Pyro4.config.HMAC_KEY=None
-        Pyro4.config.SERIALIZER="serpent"
+        Pyro4.config.SERIALIZERS_ACCEPTED.discard("pickle")
 
     def testConnectionStuff(self):
         p1=Pyro4.core.Proxy(self.objectUri)
@@ -541,7 +548,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
 
     def testSerializeConnected(self):
         # online serialization tests
-        ser=Pyro4.util.get_serializer()
+        ser=Pyro4.util.get_serializer(Pyro4.config.SERIALIZER)
         proxy=Pyro4.core.Proxy(self.objectUri)
         proxy._pyroBind()
         self.assertFalse(proxy._pyroConnection is None)
@@ -550,7 +557,6 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
         self.assertTrue(proxy2._pyroConnection is None)
         self.assertFalse(proxy._pyroConnection is None)
         self.assertEqual(proxy2._pyroUri, proxy._pyroUri)
-        self.assertEqual(proxy2._pyroSerializer, proxy._pyroSerializer)
         proxy2._pyroBind()
         self.assertFalse(proxy2._pyroConnection is None)
         self.assertFalse(proxy2._pyroConnection is proxy._pyroConnection)
@@ -567,7 +573,6 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
         self.assertFalse(proxy._pyroConnection is None)
         self.assertEqual(proxy3._pyroUri, proxy._pyroUri)
         self.assertFalse(proxy3._pyroUri is proxy._pyroUri)
-        self.assertEqual(proxy3._pyroSerializer, proxy._pyroSerializer)        
         proxy._pyroRelease()
         proxy2._pyroRelease()
         proxy3._pyroRelease()
@@ -705,14 +710,15 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
             # so 6 threads taking 0.5 seconds =~ 0.5 seconds passed
             self.assertTrue(0.4<duration<0.9)  # loose upper bound for slow jython
 
-if os.name!="java":
-    class ServerTestsMultiplexNoTimeout(ServerTestsThreadNoTimeout):
-        SERVERTYPE="multiplex"
-        COMMTIMEOUT=None
-        def testProxySharing(self):
-            pass
-        def testException(self):
-            pass
+
+class ServerTestsMultiplexNoTimeout(ServerTestsThreadNoTimeout):
+    SERVERTYPE = "multiplex"
+    COMMTIMEOUT = None
+    def testProxySharing(self):
+        pass
+    def testException(self):
+        pass
+
 
 if __name__ == "__main__":
     #import sys;sys.argv = ['', 'Test.testName']
