@@ -51,17 +51,17 @@ class TestSocketutil(unittest.TestCase):
         Pyro4.config.PREFER_IP_VERSION=4
         myip=SU.getIpAddress("")
         self.assertTrue(len(myip)>4)
-        myip=SU.getIpAddress("",workaround127=True)
+        myip=SU.getIpAddress("", workaround127=True)
         self.assertTrue(len(myip)>4)
         self.assertFalse(myip.startswith("127."))
-        self.assertEqual("127.0.0.1", SU.getIpAddress("127.0.0.1",workaround127=False))
-        self.assertNotEqual("127.0.0.1", SU.getIpAddress("127.0.0.1",workaround127=True))
+        self.assertEqual("127.0.0.1", SU.getIpAddress("127.0.0.1", workaround127=False))
+        self.assertNotEqual("127.0.0.1", SU.getIpAddress("127.0.0.1", workaround127=True))
         
     @unittest.skipUnless(has_ipv6, "ipv6 testcase")
     def testGetIP6(self):
-        self.assertTrue(":" in SU.getIpAddress("::1",ipVersion=6))
-        self.assertTrue(":" in SU.getIpAddress("",ipVersion=6))
-        self.assertTrue(":" in SU.getIpAddress("localhost",ipVersion=6))
+        self.assertTrue(":" in SU.getIpAddress("::1", ipVersion=6))
+        # self.assertTrue(":" in SU.getIpAddress("", ipVersion=6))
+        self.assertTrue(":" in SU.getIpAddress("localhost", ipVersion=6))
 
     def testGetIpVersion4(self):
         version = Pyro4.config.PREFER_IP_VERSION
@@ -214,6 +214,13 @@ class TestSocketutil(unittest.TestCase):
         self.assertEqual(SOCKNAME,s.getsockname())
         s.close()
         if os.path.exists(SOCKNAME): os.remove(SOCKNAME)
+        # unicode arg
+        SOCKNAME = unicode(SOCKNAME)
+        s=SU.createSocket(bind=SOCKNAME)
+        self.assertEqual(socket.AF_UNIX, s.family)
+        self.assertEqual(SOCKNAME,s.getsockname())
+        s.close()
+        if os.path.exists(SOCKNAME): os.remove(SOCKNAME)
         self.assertRaises(ValueError, SU.createSocket, bind=SOCKNAME, connect=SOCKNAME)
 
     def testSend(self):
@@ -324,7 +331,7 @@ class ServerCallback(object):
             raise TypeError("handleRequest expected SocketConnection parameter")
         msg = Pyro4.message.Message.recv(connection, [Pyro4.message.MSG_PING])
         if msg.type == Pyro4.message.MSG_PING:
-            msg = Pyro4.message.Message(Pyro4.message.MSG_PING, b"", msg.serializer_id, 0, msg.seq)
+            msg = Pyro4.message.Message(Pyro4.message.MSG_PING, b"ping", msg.serializer_id, 0, msg.seq)
             connection.send(msg.to_bytes())
         else:
             print("unhandled message type", msg.type)
@@ -393,7 +400,6 @@ class TestSocketServer(unittest.TestCase):
 
 
 @unittest.skipUnless(SU.hasSelect, "requires select()")
-@unittest.skipUnless(os.name!="java", "select-server not yet supported in jython")
 class TestServerDOS_select(unittest.TestCase):
 
     def setUp(self):
@@ -507,11 +513,11 @@ class TestServerDOS_threading(TestServerDOS_select):
     def setUp(self):
         super(TestServerDOS_threading, self).setUp()
         self.socket_server = SocketServer_Threadpool
-        self.orig_maxthreads = Pyro4.config.THREADPOOL_MAXTHREADS
-        Pyro4.config.THREADPOOL_MAXTHREADS = 1
+        self.orig_numthreads = Pyro4.config.THREADPOOL_SIZE
+        Pyro4.config.THREADPOOL_SIZE = 1
 
     def tearDown(self):
-        Pyro4.config.THREADPOOL_MAXTHREADS = self.orig_maxthreads
+        Pyro4.config.THREADPOOL_SIZE = self.orig_numthreads
 
 
 if __name__ == "__main__":
